Event = require('knife.knife.event')
Timer = require('knife.knife.timer')
push = require('push.push')

SPRITE_WIDTH = 32
SPRITE_HEIGHT = 36

VIRTUAL_WIDTH = 512
VIRTUAL_HEIGHT = 288

local function _GenerateFrames(atlas, frameWidth, frameHeight)
    local tiles_across = atlas:getWidth() / frameWidth
    local tiles_down = atlas:getHeight() / frameHeight
    local sprite_sheet = {}
    local frame_num = 0
    for tileY = 0, tiles_down - 1 do
        for tileX = 0, tiles_across - 1 do
            frame_num = frame_num + 1
            sprite_sheet[frame_num] = love.graphics.newQuad(tileX * frameWidth, tileY* frameHeight, frameWidth, frameHeight, atlas:getDimensions())
        end
    end
    return sprite_sheet
end

local _textures = {
    ['healer-f'] = love.graphics.newImage('images/rpgsprites1/healer_f.png'),
    ['healer-m'] = love.graphics.newImage('images/rpgsprites1/healer_m.png'),
    ['mage-f'] = love.graphics.newImage('images/rpgsprites1/mage_f.png'),
    ['mage-m'] = love.graphics.newImage('images/rpgsprites1/mage_m.png'),
    ['ninja-f'] = love.graphics.newImage('images/rpgsprites1/ninja_f.png'),
    ['ninja-m'] = love.graphics.newImage('images/rpgsprites1/ninja_m.png'),
    ['ranger-f'] = love.graphics.newImage('images/rpgsprites1/ranger_f.png'),
    ['ranger-m'] = love.graphics.newImage('images/rpgsprites1/ranger_m.png'),
    ['townfolk1-f'] = love.graphics.newImage('images/rpgsprites1/townfolk1_f.png'),
    ['townfolk1-m'] = love.graphics.newImage('images/rpgsprites1/townfolk1_m.png'),
    ['warrior-f'] = love.graphics.newImage('images/rpgsprites1/warrior_f.png'),
    ['warrior-m'] = love.graphics.newImage('images/rpgsprites1/warrior_m.png'),
}

local _frames = {}

for k,v in pairs(_textures) do
    _frames[k] = _GenerateFrames(_textures[k], SPRITE_WIDTH, SPRITE_HEIGHT)
end

displayItems = {}
keysMapped = 'abcdefghijkl'

function love.load()
    local game_width, game_height = VIRTUAL_WIDTH, VIRTUAL_HEIGHT
    local window_width, window_height = love.window.getDesktopDimensions()
    window_width, window_height = window_width*.5, window_height*.5
    local animation_gap = SPRITE_WIDTH

    push:setupScreen(game_width, game_height, window_width, window_height, {fullscreen = false, resizeable = false})

    local char_x, char_y = animation_gap, animation_gap
    local curr_item = 0
    for k, character in pairs(_textures) do
        curr_item = curr_item + 1
        local entry =  {
            texture = _textures[k],
            frames = _frames[k],
            x = char_x,
            y = char_y,
            width = SPRITE_WIDTH,
            height = SPRITE_HEIGHT,
            currFrame = 1,
        }
        table.insert(displayItems, entry)
        entry.incrementFrame = function(self)
            local newFrame = self.currFrame + 1
            if newFrame > #self.frames then newFrame = 1 end
            self.currFrame = newFrame
        end
        Event.on('keypressed_'..string.sub(keysMapped, curr_item, curr_item),
            function()
                entry:incrementFrame() 
            end
        )
        char_x = char_x + SPRITE_WIDTH + animation_gap
        if char_x > game_width - animation_gap - SPRITE_WIDTH then
            char_x = animation_gap
            char_y = char_y + SPRITE_HEIGHT + animation_gap
        end
    end
end

function love.update(dt)
    Timer.update(dt)
end

function love.draw()
    push:start()
    for _,item in ipairs(displayItems) do
        love.graphics.setColor(1, 1, 1, 1)
        love.graphics.draw(item.texture, item.frames[item.currFrame], item.x, item.y)
    end
    push:finish()
end

function love.keypressed(key)
    if key == 'escape' then
        love.event.quit()
    elseif string.find(keysMapped, key) then
        Event.dispatch('keypressed_' .. key)
    elseif key == 'lshift' then
        Timer.every(0.5, function()
            for _,item in ipairs(displayItems) do
                item:incrementFrame()
            end
        end)
    elseif key == 'rshift' then
        Timer.clear()
    end
end
